"""This module is for the translation mapping data of the
following document:

Document file: animation_finish.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# animation_finish interface": "# animation_finish インターフェイス",
    ##################################################
    "This page explains the `animation_finish` method interface.": "このページでは`animation_finish`メソッドのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `animation_finish` interface finishes all running animations and sets the last attribute values of each animation setting.": "`animation_finish`インターフェイスでは実行されている全てのアニメーションにアニメーションの各属性の最終値を設定しアニメーションを終了させます。",  # noqa
    ##################################################
    "This interface exists in the instances that have the animation interfaces (such as the `animation_x`\\, `animation_move`).": "このインターフェイスは`animation_x`や`animation_move`などのアニメーション関係のインターフェイスを持つクラスのインスタンス上に存在します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The following example sets the click event to the rectangle. If you click the rectangle, the x-coordinate animation starts. After 2 seconds have passed, the x-coordinate animation finishes and sets the last x-coordinate value of the animation.": "以下のコード例では四角にクリックイベントを設定しています。四角をクリックするとX座標のアニメーションが開始するようになっています。アニメーション開始後に2秒経過したらアニメーションは終了し、最終値の座標が設定されます。",  # noqa
    ##################################################
    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.animation_x(\n        x=300,\n        duration=5000,\n    ).start()\n\n    options_: _RectOptions = {"rectangle": rectangle}\n    ap.Timer(\n        on_timer,\n        delay=2000,\n        repeat_count=1,\n        options=options_,\n    ).start()\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that the timer calls when its ends.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options["rectangle"]\n    rectangle.animation_finish()\n\n\nap.Stage(\n    stage_width=400, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(on_click)\n\nap.save_overall_html(dest_dir_path="animation_finish_basic_usage/")\n```': '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.animation_x(\n        x=300,\n        duration=5000,\n    ).start()\n\n    options_: _RectOptions = {"rectangle": rectangle}\n    ap.Timer(\n        on_timer,\n        delay=2000,\n        repeat_count=1,\n        options=options_,\n    ).start()\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that the timer calls when its ends.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options["rectangle"]\n    rectangle.animation_finish()\n\n\nap.Stage(\n    stage_width=400, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(on_click)\n\nap.save_overall_html(dest_dir_path="animation_finish_basic_usage/")\n```',  # noqa
    ##################################################
    "## animation_finish API": "## animation_finish API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Finish all animations (set the animation last value to each attribute).<hr>": "全てのアニメーションを終了します（各属性にアニメーション終了時の値を設定します）。<hr>",  # noqa
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n...\n>>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options["rectangle"]\n...     rectangle.animation_finish()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.animation_x(\n...     x=100,\n...     duration=1500,\n...     easing=ap.Easing.EASE_OUT_QUINT,\n... ).start()\n>>> options: RectOptions = {"rectangle": rectangle}\n>>> ap.Timer(on_timer, delay=750, options=options).start()\n```': '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n...\n>>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options["rectangle"]\n...     rectangle.animation_finish()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.animation_x(\n...     x=100,\n...     duration=1500,\n...     easing=ap.Easing.EASE_OUT_QUINT,\n... ).start()\n>>> options: RectOptions = {"rectangle": rectangle}\n>>> ap.Timer(on_timer, delay=750, options=options).start()\n```',  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: why_apysc_doesnt_use_python_builtin_data_type.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Why the apysc library doesn't use the Python built-in data type": "# 何故 apysc ライブラリではPythonのビルトインのデータ型を使用していないのか",  # noqa
    ##################################################
    "This page explains why the apysc library doesn't use the Python built-in data type, such as `int`\\, `float`\\, `bool`\\, `list`\\. And why the apysc library uses the apysc data type like the `Int`\\, `Number`\\, `Array` instead.": "このページでは apysc ライブラリが何故`int`や`float`、`bool`、`list`などのPythonビルトインのデータ型を使用していないのかについて説明します。また、何故それらの代わりに`Int`や`Number`、`Array`などのデータの型を使用しているのかについても説明します。",  # noqa
    ##################################################
    "## apysc needs to convert Python to JavaScript and track variables change": "## apysc ではPythonをJavaScriptへと変換する必要があり、変数の変化を追う必要があります",  # noqa
    ##################################################
    "The apysc library needs to track variable creation and update to convert the Python code to JavaScript. For this reason, apysc using the original data types, such as `Int`\\, `Number` (`Float`), `String`\\, `Boolean`\\, `Array`\\, and `Dictionary`.": "apysc ライブラリではPythonのコードをJavaScriptへと変換するために変数の生成や更新などの処理を内部で追う必要があります。この理由から、 apysc では`Int`や`Number`（`Float`）、`String`、`Boolean`、`Array`、`Dictionary`などの独自の型を設けてそちらを使用しています。",  # noqa
    ##################################################
    "Occasionally, these are unnecessary to create HTML. Still, these types become essential when you use the asynchronous function, such as the event handler.": "場合によってはHTMLの生成処理でこれらの型の利用が不要な場合もありますが、イベントハンドラなどの非同期な処理を使う場合などには利用が必要になってきます。",  # noqa
    ##################################################
    "The apysc library automatically sets each variable's names and uses them when exporting the HTML and JavaScript files. It also tracks variables creation and updating and applies them to the exported JavaScript.": "apysc ライブラリではそれらの型の内部で自動的に変数名を割り振り、HTMLやJavaScriptのファイルを出力する際にそれらの変数名を使用します。また、変数の生成や更新などの内容も出力されるJavaScriptのコードに反映されます。",  # noqa
    ##################################################
    "Using the Python built-in data type, these variables' values become fixed (the apysc doesn't apply asynchronous function's changes).": "もしPythonビルトインの型を使った場合、これらの値JavaScript上では固定値で設定されます（apysc上では非同期の関数などでの変数の変更が反映されなくなります）。",  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: graphics_draw_rect.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Graphics draw_rect interface": "# Graphics クラスの draw_rect インターフェイス",
    ##################################################
    "This page explains the `Graphics` class `draw_rect` method interface.": "このページでは`Graphics`クラスの`draw_rect`メソッドのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "`draw_rect` interface draws vector rectangle graphics.": "`draw_rect`インターフェイスは四角のベクターグラフィックスを描画します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "`draw_rect` interface has `x`, `y`, `width`, and `height` arguments. `x` and `y` are rectangle coordinates settings, and `width` and `height` will determine rectangle size.": "`draw_rect`インターフェイスは`x`、`y`、`width`、`height`の各引数を持っています。`x`と`y`は四角の座標を設定し、`width`と`height`は四角のサイズを決定します。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=200, stage_height=150, stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nsprite.graphics.draw_rect(x=50, y=50, width=100, height=50)\n\nap.save_overall_html(dest_dir_path="graphics_draw_rect_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=200, stage_height=150, stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nsprite.graphics.draw_rect(x=50, y=50, width=100, height=50)\n\nap.save_overall_html(dest_dir_path="graphics_draw_rect_basic_usage/")\n```',  # noqa
    ##################################################
    "The previous script draws horizontal rectangle graphics.": "前述のコードでは横長の四角を描画しています。",  # noqa
    ##################################################
    "Notes: `begin_fill` call (fill color setting) is necessary before `draw_rect` interface call. If you skip it, it displays nothing on stage.": "特記事項: `draw_rect`のインターフェイスを呼ぶ前に`begin_fill`メソッド（塗りの設定のインターフェイス）を呼んでおく必要があります。もし`begin_fill`の呼び出しがされていない場合画面上に四角が表示されません。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color="#333", stage_width=200, stage_height=150, stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.draw_rect(x=50, y=50, width=100, height=50)\n\nap.save_overall_html(dest_dir_path="graphics_draw_rect_basic_usage_skipped_begin_fill/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color="#333", stage_width=200, stage_height=150, stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.draw_rect(x=50, y=50, width=100, height=50)\n\nap.save_overall_html(dest_dir_path="graphics_draw_rect_basic_usage_skipped_begin_fill/")\n```',  # noqa
    ##################################################
    "## Rectangle instance": "## Rectangle インスタンス",
    ##################################################
    "`draw_rect` interface will return `Rectangle` instance. You can update each setting or bind events.": "`draw_rect`インターフェイスは`Rectangle`クラスのインスタンスを変逆します。そのインスタンスに対して各属性の更新やイベントハンドラの登録などを行うことができます。",  # noqa
    ##################################################
    "For instance, the following script sets the mouse event to `Rectangle` and updates the x position in the handler (`on_click`).": "例えば以下のコードでは`Rectangle`のインスタンスに対してクリックのマウスイベントを登録し、`on_click`のハンドラ内でX座標を更新しています。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Created event instance.\n    options : dict\n        Optional arguments.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.x = ap.Int(100)\n\n\nap.Stage(\n    background_color="#333", stage_width=200, stage_height=150, stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(on_click)\n\nap.save_overall_html(dest_dir_path="graphics_draw_rect_rectangle/")\n```': '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Created event instance.\n    options : dict\n        Optional arguments.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.x = ap.Int(100)\n\n\nap.Stage(\n    background_color="#333", stage_width=200, stage_height=150, stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(on_click)\n\nap.save_overall_html(dest_dir_path="graphics_draw_rect_rectangle/")\n```',  # noqa
    ##################################################
    "If you click rectangle, the handler changes the x position to 100.": "四角をクリックしてみると、ハンドラはX座標を100pxの位置に変更します。",  # noqa
    ##################################################
    "## draw_rect API": "## draw_rect API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Draw a rectangle vector graphics.<hr>": "ベクターグラフィックスの四角を描画します。<hr>",
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `x`: Int or int": "- `x`: Int or int",
    ##################################################
    "  - X position to start drawing.": "  - 描画を開始する位置のX座標。",
    ##################################################
    "- `y`: Int or int": "- `y`: Int or int",
    ##################################################
    "  - Y position to start drawing.": "  - 描画を開始する位置のY座標。",
    ##################################################
    "- `width`: Int or int": "- `width`: Int or int",
    ##################################################
    "  - Rectangle width.": "  - 四角の幅。",
    ##################################################
    "- `height`: Int or int": "- `height`: Int or int",
    ##################################################
    "  - Rectangle height.": "  - 四角の高さ。",
    ##################################################
    "- `variable_name_suffix`: str, default ''": "- `variable_name_suffix`: str, default ''",  # noqa
    ##################################################
    "  - A JavaScript variable name suffix string. This setting is sometimes useful for JavaScript's debugging.": "  - JavaScript上の変数のサフィックスの設定です。この設定はJavaScriptのデバッグ時に役立つことがあります。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `rectangle`: Rectangle": "- `rectangle`: Rectangle",
    ##################################################
    "  - Created rectangle.": "  - 生成された四角。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\"#0af\")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> rectangle.x\nInt(50)\n\n>>> rectangle.width\nInt(50)\n\n>>> rectangle.fill_color\nString('#00aaff')\n```": "```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\"#0af\")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> rectangle.x\nInt(50)\n\n>>> rectangle.width\nInt(50)\n\n>>> rectangle.fill_color\nString('#00aaff')\n```",  # noqa
}



"""Class implementation for the parallel animation mix-in.
"""

from typing import List
from typing import Union

from typing_extensions import final

from apysc._animation.animation_base import AnimationBase
from apysc._animation.animation_mixins import AnimationMixIns
from apysc._animation.animation_parallel import AnimationParallel
from apysc._animation.easing import Easing
from apysc._type.int import Int
from apysc._validation import arg_validation_decos


class AnimationParallelMixIn(AnimationMixIns):
    @final
    @arg_validation_decos.is_animations(arg_position_index=1)
    @arg_validation_decos.is_integer(arg_position_index=2)
    @arg_validation_decos.num_is_gt_zero(arg_position_index=2)
    @arg_validation_decos.is_integer(arg_position_index=3)
    @arg_validation_decos.is_easing(arg_position_index=4)
    def animation_parallel(
        self,
        *,
        animations: List[AnimationBase],
        duration: Union[int, Int] = 3000,
        delay: Union[int, Int] = 0,
        easing: Easing = Easing.LINEAR
    ) -> AnimationParallel:
        """
        Set the parallel animation setting.

        Notes
        -----
        - To start this animation, you need to call the `start` method of
            the returned instance.
        - The `animations` argument can't contains the `AnimationParallel`
            instance.
        - This interface ignores the duration, delay, and easing
            arguments in the `animations` argument
            (this interface uses self-arguments instead).

        Raises
        ------
        ValueError
            - If the animations' target is not this instance.
            - If there are changed duration, delay, or easing
                animation settings in the `animations` list.

        Parameters
        ----------
        animations : list of AnimationBase
            Target animation settings.
        duration : Int or int, default 3000
            Milliseconds before an animation ends.
        delay : Int or int, default 0
            Milliseconds before an animation starts.
        easing : Easing, default Easing.LINEAR
            Easing setting.

        Returns
        -------
        animation_parallel : AnimationParallel
            Created animation setting instance.

        References
        ----------
        - Animation interfaces duration setting
            - https://simon-ritchie.github.io/apysc/en/animation_duration.html
        - Animation interfaces delay setting
            - https://simon-ritchie.github.io/apysc/en/animation_delay.html
        - Each animation interface return value
            - https://simon-ritchie.github.io/apysc/en/animation_return_value.html  # noqa
        - Sequential animation setting
            - https://simon-ritchie.github.io/apysc/en/sequential_animation.html
        - animation_parallel interface
            - https://simon-ritchie.github.io/apysc/en/animation_parallel.html
        - Easing enum
            - https://simon-ritchie.github.io/apysc/en/easing_enum.html

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color="#0af")
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50
        ... )
        >>> _ = rectangle.animation_parallel(
        ...     animations=[
        ...         rectangle.animation_x(x=100),
        ...         rectangle.animation_fill_color(fill_color="#f0a"),
        ...         rectangle.animation_fill_alpha(alpha=0.5),
        ...     ],
        ...     duration=1500,
        ...     easing=ap.Easing.EASE_OUT_QUINT,
        ... ).start()
        """
        animation_parallel: AnimationParallel = AnimationParallel(
            target=self,
            animations=animations,
            duration=duration,
            delay=delay,
            easing=easing,
        )
        return animation_parallel



"""This module is for the translation mapping data of the
following document:

Document file: display_object_mouse_event.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# DisplayObject class mouse event binding interfaces": "# DisplayObject クラスのマウスイベント設定の各インターフェイス",  # noqa
    ##################################################
    "This page explains the `DisplayObject` class mouse event binding interfaces.": "このページでは`DisplayObject`クラスのマウスイベントの登録の各インターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interfaces are these?": "## 各インターフェイスの概要",
    ##################################################
    "Each `DisplayObject` instance has the mouse event binding interfaces, like the click, mouse over, mouse move.": "各`DisplayObject`のインスタンスはクリックやマウスオーバーなどのマウスイベント登録用の各インターフェイスを持っています。",  # noqa
    ##################################################
    "These interfaces can bind the mouse event to a `DisplayObject` instance. So, for instance, you can assign any function to handle when a click of `DisplayObject` instance.": "これらのインターフェイスは`DisplayObject`にマウスイベントを設定でき、例えばクリック時に実行したい関数などを登録することができます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "You can bind event handler (callable) with each interface, like the `click`\\, `mouseover`\\.": "`click`や`mouseover`などの各インターフェイスで任意のイベントハンドラ（Callableオブジェクト）を登録することができます。",  # noqa
    ##################################################
    "The following example binds the click event handler, and if you click the rectangle, the fill color is changed.": "以下のコード例ではクリックのイベントハンドラを設定しており、四角をクリックすると色が変わるようにしています。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.fill_color = ap.String("#f0a")\n\n\nap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(on_click)\n\nap.save_overall_html(dest_dir_path="display_object_mouse_event_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.fill_color = ap.String("#f0a")\n\n\nap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(on_click)\n\nap.save_overall_html(dest_dir_path="display_object_mouse_event_basic_usage/")\n```',  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "For more details, please see the following pages:": "詳細については以下の各ページをご確認ください:",
    ##################################################
    "- [Basic mouse event interfaces](mouse_event_basic.md)": "- [基本的なマウスイベントの各インターフェイス](jp_mouse_event_basic.md)",  # noqa
    ##################################################
    "- [click interface](click.md)": "- [click インターフェイス](jp_click.md)",
    ##################################################
    "- [mousedown and mouseup interfaces](mousedown_and_mouseup.md)": "- [mousedown と mouseup のインターフェイス](jp_mousedown_and_mouseup.md)",  # noqa
    ##################################################
    "- [mouseover and mouseout interfaces](mouseover_and_mouseout.md)": "- [mouseover と mouseout のインターフェイス](jp_mouseover_and_mouseout.md)",  # noqa
    ##################################################
    "- [mousemove interface](mousemove.md)": "- [mousemove インターフェイス](jp_mousemove.md)",
}



"""This module is for the translation mapping data of the
following document:

Document file: add_debug_info_setting.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# add_debug_info_setting decorator interface": "# add_debug_info_setting のデコレーターのインターフェイス",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `add_debug_info_setting` decorator interface sets the debug information settings to a target function or method.": "`add_debug_info_setting`のデコレーターのインターフェイスは対象の関数もしくはメソッドへとデバッグ情報の設定を行います。",  # noqa
    ##################################################
    "A decorated function or method exports debug information when you enable the debug mode setting.": "デコレーターが設定された関数もしくはメソッドでは、デバッグモードを有効にした後の処理では各デバッグ情報を出力するようになります。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "It is necessary to set the debug mode setting with the `ap.set_debug_mode()` function at the first position.": "処理の有効にするために最初に`ap.set_debug_mode()`関数でデバッグモードの設定を行っておく必要があります。",  # noqa
    ##################################################
    "After that, You can set the `@ap.add_debug_info_setting` decorator settings to any function or method.": "その後、任意の各関数やメソッドに`@ap.add_debug_info_setting`のデコレーターの設定を追加することができます。",  # noqa
    ##################################################
    "The `@ap.add_debug_info_setting` needs to specify the `module_name` (this value becomes `__name__`).": "`@ap.add_debug_info_setting`の関数は`module_name`引数の指定が必要になります（この引数は基本的に`__name__`の値となります）。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\n\ndef _main() -> None:\n    """The entry point of this project."""\n    ap.Stage(\n        background_color="#333",\n        stage_width=150,\n        stage_height=150,\n        stage_elem_id="stage",\n    )\n    ap.set_debug_mode()\n    _draw_rectangle(x=50, y=50)\n    ap.save_overall_html(dest_dir_path="add_debug_info_setting_basic_usage/")\n\n\n@ap.add_debug_info_setting(module_name=__name__)\ndef _draw_rectangle(*, x: int, y: int) -> None:\n    """\n    Draw a rectangle with the given coordinates and sprite\n    container class.\n\n    Parameters\n    ----------\n    x : int\n        X-coordinate of the rectangle.\n    y : int\n        Y-coordinate of the rectangle.\n    """\n    _: MySprite = MySprite(x=x, y=y)\n\n\nclass MySprite(ap.Sprite):\n    @ap.add_debug_info_setting(module_name=__name__)\n    def __init__(self, *, x: int, y: int) -> None:\n        """\n        My rectangle\'s sprite container class.\n\n        Parameters\n        ----------\n        x : int\n            X-coordinate of the rectangle.\n        y : int\n            Y-coordinate of the rectangle.\n        """\n        super(MySprite, self).__init__()\n        self.graphics.begin_fill(color="#0af")\n        self.graphics.draw_rect(x=x, y=y, width=50, height=50)\n\n\nif __name__ == "__main__":\n    _main()\n```': '```py\n# runnable\nimport apysc as ap\n\n\ndef _main() -> None:\n    """The entry point of this project."""\n    ap.Stage(\n        background_color="#333",\n        stage_width=150,\n        stage_height=150,\n        stage_elem_id="stage",\n    )\n    ap.set_debug_mode()\n    _draw_rectangle(x=50, y=50)\n    ap.save_overall_html(dest_dir_path="add_debug_info_setting_basic_usage/")\n\n\n@ap.add_debug_info_setting(module_name=__name__)\ndef _draw_rectangle(*, x: int, y: int) -> None:\n    """\n    Draw a rectangle with the given coordinates and sprite\n    container class.\n\n    Parameters\n    ----------\n    x : int\n        X-coordinate of the rectangle.\n    y : int\n        Y-coordinate of the rectangle.\n    """\n    _: MySprite = MySprite(x=x, y=y)\n\n\nclass MySprite(ap.Sprite):\n    @ap.add_debug_info_setting(module_name=__name__)\n    def __init__(self, *, x: int, y: int) -> None:\n        """\n        My rectangle\'s sprite container class.\n\n        Parameters\n        ----------\n        x : int\n            X-coordinate of the rectangle.\n        y : int\n            Y-coordinate of the rectangle.\n        """\n        super(MySprite, self).__init__()\n        self.graphics.begin_fill(color="#0af")\n        self.graphics.draw_rect(x=x, y=y, width=50, height=50)\n\n\nif __name__ == "__main__":\n    _main()\n```',  # noqa
    ##################################################
    "The exported HTML contains the following debug information (function and method calling information):": "出力されたHTMLには以下のように関数やメソッドの呼び出しなどのデバッグ情報が含まれるようになります:",  # noqa
    ##################################################
    "```\n...\n  //////////////////////////////////////////////////////////////////////\n  // [_draw_rectangle 1] started.\n  // module name: __main__\n  // Keyword arguments: {'x': 50, 'y': 50}\n    //////////////////////////////////////////////////////////////////////\n    // [__init__ 1] started.\n    // module name: __main__\n    // class: MySprite\n    // Positional arguments: [Sprite('')]\n    // Keyword arguments: {'x': 50, 'y': 50}\n...\n```": "```\n...\n  //////////////////////////////////////////////////////////////////////\n  // [_draw_rectangle 1] started.\n  // module name: __main__\n  // Keyword arguments: {'x': 50, 'y': 50}\n    //////////////////////////////////////////////////////////////////////\n    // [__init__ 1] started.\n    // module name: __main__\n    // class: MySprite\n    // Positional arguments: [Sprite('')]\n    // Keyword arguments: {'x': 50, 'y': 50}\n...\n```",  # noqa
    ##################################################
    "## Notes of the mypy setting": "## mypy設定に対する特記事項",
    ##################################################
    "This decorator settings currently raise a mypy error. To avoid this error, please set the `--disable-error-code misc` option.": "このデコレーター設定は現在mypy上でエラーが発生します。ごのエラーを避けるためには`--disable-error-code misc`オブションの指定の追加を検討してください。",  # noqa
    ##################################################
    "See also:": "参考資料:",
    ##################################################
    "- [Recommended type annotation checker settings](recommended_type_checker_settings.md)": "- [推奨される型アノテーションのチェック設定](jp_recommended_type_checker_settings.md)",  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "- [set_debug_mode interface](set_debug_mode.md)": "- [set_debug_mode インターフェイス](jp_set_debug_mode.md)",  # noqa
    ##################################################
    "- [unset_debug_mode interface](unset_debug_mode.md)": "- [unset_debug_mode インターフェイス](jp_unset_debug_mode.md)",  # noqa
}



"""Class implementation for a handler.
"""

from dataclasses import dataclass
from typing import Any
from typing import Callable
from typing import Generic
from typing import List
from typing import TypeVar

from apysc._event.event import Event
from apysc._event.mouse_event_type import MouseEventType
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.variable_name_mixin import VariableNameMixIn
from apysc._validation import arg_validation_decos

_Handler = Callable[[Any, Any], None]
_EventType = TypeVar("_EventType", bound=Event)


@dataclass
class HandlerData(Generic[_EventType]):
    handler: Callable[[_EventType, Any], None]
    options: Any


def get_handler_name(*, handler: _Handler, instance: Any) -> str:
    """
    Get a handler name.

    Parameters
    ----------
    handler : _Handler
        Target handler.
    instance : VariableNameMixIn
        Instance to bind target handler.

    Returns
    -------
    handler_name : str
        Handler name (method path + class name (if handler is method)
        + function or method name) + instance's variable name.
    """
    from apysc._callable import callable_util
    from apysc._event import handler_circular_calling_util as circ_util
    from apysc._validation.variable_name_validation import (
        validate_variable_name_interface_type,
    )

    class_name: str = callable_util.get_method_class_name(method=handler)
    if class_name != "":
        class_name = f"{class_name}_"
    handler_name: str = (
        f"{handler.__module__}{class_name}{handler.__name__}"  # type: ignore
    )
    handler_name = handler_name.replace(".", "_")
    instance_: VariableNameMixIn = validate_variable_name_interface_type(
        instance=instance
    )
    handler_name += f"_{instance_.variable_name}"
    if circ_util.is_handler_circular_calling(handler_name=handler_name):
        return circ_util.get_prev_handler_name(handler_name=handler_name)
    return handler_name


@arg_validation_decos.is_event(arg_position_index=2)
@add_debug_info_setting(module_name=__name__)
def append_handler_expression(
    *,
    handler_data: HandlerData,
    handler_name: str,
    e: Event,
    in_handler_head_expression: str = "",
) -> None:
    """
    Append a handler's expression.

    Parameters
    ----------
    handler_data : HandlerData
        Target handler's data to append.
    handler_name : str
        Target handler's name.
    e : Event
        Created event instance.
    in_handler_head_expression : str, default ''
        Optional expression to be added at the handler function's
        head position.
    """
    import apysc as ap
    from apysc._event.handler_circular_calling_util import is_handler_circular_calling
    from apysc._expression.event_handler_scope import HandlerScope
    from apysc._expression.indent_num import Indent
    from apysc._type import revert_mixin
    from apysc._validation.variable_name_validation import (
        validate_variable_name_interface_type,
    )

    variables: List[Any] = [*handler_data.options.values()]
    snapshot_name: str = revert_mixin.make_variables_snapshots(variables=variables)
    instance: VariableNameMixIn = validate_variable_name_interface_type(instance=e.this)

    with HandlerScope(handler_name=handler_name, instance=instance):
        is_handler_circular_calling_: bool = is_handler_circular_calling(
            handler_name=handler_name
        )
        if not is_handler_circular_calling_:
            expression: str = f"function {handler_name}({e.variable_name}) {{"
            ap.append_js_expression(expression=expression)
            with Indent():
                _append_in_handler_head_expression(
                    in_handler_head_expression=in_handler_head_expression
                )
                handler_data.handler(e, handler_data.options)
            ap.append_js_expression(expression="}")

    revert_mixin.revert_variables(snapshot_name=snapshot_name, variables=variables)


@add_debug_info_setting(module_name=__name__)
def _append_in_handler_head_expression(*, in_handler_head_expression: str) -> None:
    """
    Append an in-handler head expression if it is not blank.

    Parameters
    ----------
    in_handler_head_expression : str
        Optional expression to be added at the handler function's
        head position if it is not blank.
    """
    import apysc as ap

    if in_handler_head_expression == "":
        return
    ap.append_js_expression(expression=in_handler_head_expression)


@add_debug_info_setting(module_name=__name__)
def append_unbinding_expression(
    *, this: VariableNameMixIn, handler_name: str, mouse_event_type: MouseEventType
) -> None:
    """
    Append event unbinding expression.

    Parameters
    ----------
    this : VariableNameMixIn
        Instance that event is binded.
    handler_name : str
        Target handler's name.
    mouse_event_type : MouseEventType
        Event type to unbind.
    """
    import apysc as ap
    from apysc._validation import event_validation

    event_validation.validate_event_type(mouse_event_type=mouse_event_type)
    expression: str = (
        f'{this.variable_name}.off("{mouse_event_type.value}", ' f"{handler_name});"
    )
    ap.append_js_expression(expression=expression)


@add_debug_info_setting(module_name=__name__)
def append_unbinding_all_expression(
    *, this: VariableNameMixIn, mouse_event_type: MouseEventType
) -> None:
    """
    Append all events unbinding expression.

    Parameters
    ----------
    this : VariableNameMixIn
        Instance that events are binded.
    mouse_event_type : MouseEventType
        Event type to unbind.
    """
    import apysc as ap
    from apysc._validation import event_validation

    event_validation.validate_event_type(mouse_event_type=mouse_event_type)
    expression: str = f'{this.variable_name}.off("{mouse_event_type.value}");'
    ap.append_js_expression(expression=expression)



"""This module is for the translation mapping data of the
following document:

Document file: for.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# For class": "# For クラス",
    ##################################################
    "This page explains the `For` class.": "このページでは`For`クラスについて説明します。",
    ##################################################
    "Before reading on, maybe it is helpful to read the following page (the apysc uses the `For` class for the same reason for each data type):": "このページを読み進める前に以下のページを事前に確認しておくと役に立つかもしれません（apyscでは`For`クラスを各データ型のクラスと同じ理由で使用しています）。",  # noqa
    ##################################################
    "- [Why the apysc library doesn't use the Python built-in data type](why_apysc_doesnt_use_python_builtin_data_type.md)": "- [なぜapyscではPythonのビルトインのデータの型を使用していないのか](jp_why_apysc_doesnt_use_python_builtin_data_type.md)",  # noqa
    ##################################################
    "## What is the For class?": "## For クラスの概要",
    ##################################################
    "The `For` class is the apysc for loop class. It behaves like the Python built-in `for` keyword.": "`For`クラスはapyscのループ制御用のクラスです。このクラスはPythonのビルトインの`for`のキーワードのように動作します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `For` class requires using the `with` statement. The `as` keyword value becomes the `Int` type index (an `i` variable)  or `String` type key.": "`For`クラスは`with`ステートメントと一緒に使用する必要があります。`as`のキーワードの値は`Int`型のインデックス（`i`の変数）もしくは`String`型のキーになります。",  # noqa
    ##################################################
    "The following example draws the three rectangles in the `with For` block:": "以下のコード例では3つの四角を`with For`のブロック内で描画しています:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=350, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\n\narr: ap.Array[int] = ap.Array(range(3))\ni: ap.Int\nwith ap.For(arr) as i:\n    sprite.graphics.draw_rect(x=i * 100 + 50, y=50, width=50, height=50)\n\nap.save_overall_html(dest_dir_path="for_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=350, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\n\narr: ap.Array[int] = ap.Array(range(3))\ni: ap.Int\nwith ap.For(arr) as i:\n    sprite.graphics.draw_rect(x=i * 100 + 50, y=50, width=50, height=50)\n\nap.save_overall_html(dest_dir_path="for_basic_usage/")\n```',  # noqa
    ##################################################
    "The `For` class constructor's first argument accepts an `Array` or `Dictionary` value. If you pass a `Dictionary` value, the `as` keyword value becomes a `String` value, not `Int`\\.": "`For`クラスの第一引数は`Array`もしくは`Dictionary`クラスの値を受け付けます。もしも`Dictionary`の値を指定した場合、`as`のキーワードの値は`Int`の代わりに`String`の値になります。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=250, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\n\ndict_val: ap.Dictionary = ap.Dictionary(\n    {"magenta": ap.String("#f0a"), "cyan": ap.String("#0af")}\n)\nkey: ap.String\nwith ap.For(dict_val) as key:\n    color: ap.String = dict_val[key]\n    sprite.graphics.begin_fill(color=color)\n    condition_1: ap.Boolean = key == "magenta"\n    condition_2: ap.Boolean = key == "cyan"\n    with ap.If(condition_1):\n        sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n    with ap.Elif(condition_2):\n        sprite.graphics.draw_rect(x=150, y=50, width=50, height=50)\n\nap.save_overall_html(dest_dir_path="for_basic_usage_with_dict/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=250, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\n\ndict_val: ap.Dictionary = ap.Dictionary(\n    {"magenta": ap.String("#f0a"), "cyan": ap.String("#0af")}\n)\nkey: ap.String\nwith ap.For(dict_val) as key:\n    color: ap.String = dict_val[key]\n    sprite.graphics.begin_fill(color=color)\n    condition_1: ap.Boolean = key == "magenta"\n    condition_2: ap.Boolean = key == "cyan"\n    with ap.If(condition_1):\n        sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n    with ap.Elif(condition_2):\n        sprite.graphics.draw_rect(x=150, y=50, width=50, height=50)\n\nap.save_overall_html(dest_dir_path="for_basic_usage_with_dict/")\n```',  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "- [Each branch instruction class's scope variables reverting setting](branch_instruction_variables_reverting_setting.md)": "- [分岐条件の各クラスのスコープ内変数の復元設定](jp_branch_instruction_variables_reverting_setting.md)",  # noqa
    ##################################################
    "  - Notes: the `For` class also has the same arguments and behaves in the same way.": "  - 特記事項: `For`クラスは同じ各引数を持っており、同じように動作します。",  # noqa
    ##################################################
    "## For API": "## For API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "A class to append for the (loop) expression.<hr>": "ループのfor文の表現を追加するためのクラスです。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `arr_or_dict`: Array or Dictionary": "- `arr_or_dict`: Array or Dictionary",
    ##################################################
    "  - Array or Dictionary instance to iterate.": "  - ループで使用するためのArray もしくは Dictionary クラスのインスタンス。",  # noqa
    ##################################################
    "- `locals_`: dict or None, default None": "- `locals_`: dict or None, default None",  # noqa
    ##################################################
    "  - Current scope's local variables. Set locals() value to this argument. If specified, this interface reverts all local scope VariableNameMixIn variables (like Int, Sprite) at the end of a `For` scope. This setting is useful when you don't want to update each variable by implementing the `For` scope.": "  - 現在のスコープのローカル変数。設定する場合にはlocals()関数の値をこの引数に指定してください。もし設定された場合にはこのいんたーふぇいろは`For`クラスによるスコープの終わりにVariableNameMixInを継承した各変数（IntやSpriteなど）の値を元の状態に復元します。この設定はもしも変数の値を`For`のスコープ内で更新したくない場合などに便利な時があります。",  # noqa
    ##################################################
    "- `globals_`: dict or None, default None": "- `globals_`: dict or None, default None",  # noqa
    ##################################################
    "  - Current scope's global variables. Set globals() value to this argument. This setting works the same way as the locals_ argument.": "  - 現在のスコープの各グローバル変数。設定する場合にはglobal()関数の値をこの引数に指定してください。この設定はlocals_引数と同じように動作します。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array(range(3))\n>>> with ap.For(arr) as i:\n...     ap.trace("Loop index is:", i)\n...\n```': '```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array(range(3))\n>>> with ap.For(arr) as i:\n...     ap.trace("Loop index is:", i)\n...\n```',  # noqa
}



"""Class implementation for mouse wheel event.
"""

from typing_extensions import final

from apysc._event.event import Event
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._type.variable_name_mixin import VariableNameMixIn


class WheelEvent(Event):
    """
    Mouse wheel event class.
    """

    @final
    @add_debug_info_setting(module_name=__name__)
    def __init__(self, *, this: VariableNameMixIn) -> None:
        """
        Mouse wheel event class.

        Parameters
        ----------
        this : VariableNameMixIn
            Instance will be binded this event.

        Notes
        -----
        Not supported each SVG elements' mouse wheel event currently, only
        supported document (overall screen) mouse wheel.
        """
        from apysc._expression import var_names

        super(WheelEvent, self).__init__(this=this, type_name=var_names.WHEEL_EVENT)

    @property
    @add_debug_info_setting(module_name=__name__)
    def delta_x(self) -> Int:
        """
        Horizontal mouse wheel value.

        Returns
        -------
        delta_x : Int
            Delta x value.
        """
        import apysc as ap

        delta_x: ap.Int = ap.Int(0)
        self._append_delta_x_getter_expression(delta_x=delta_x)
        return delta_x

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_delta_x_getter_expression(self, *, delta_x: Int) -> None:
        """
        Append delta_x getter property's expression.

        Parameters
        ----------
        delta_x : Int
            Target delta x value.
        """
        import apysc as ap

        expression: str = f"{delta_x.variable_name} = " f"{self.variable_name}.deltaX;"
        ap.append_js_expression(expression=expression)

    @property
    @add_debug_info_setting(module_name=__name__)
    def delta_y(self) -> Int:
        """
        Vertical mouse wheel value.

        Returns
        -------
        delta_y : Int
            Delta y value.
        """
        import apysc as ap

        delta_y: ap.Int = ap.Int(0)
        self._append_delta_y_getter_expression(delta_y=delta_y)
        return delta_y

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_delta_y_getter_expression(self, *, delta_y: Int) -> None:
        """
        Append delta_y getter property's expression.

        Parameters
        ----------
        delta_y : Int
            Target delta y value.
        """
        import apysc as ap

        expression: str = f"{delta_y.variable_name} = " f"{self.variable_name}.deltaY;"
        ap.append_js_expression(expression=expression)
